@@ -251,12 +251,10 @@ def collect_changefile_stats(
 def calculate_stats(
     user, uname, changeset, version, tags, osm_type, timestamp, osm_obj_nodes=None
 ):
-
     if hashtags:  # intersect with changesets
         if (
             len(processed_changesets) > 0
         ):  # make sure there are changesets to intersect if not meaning hashtag changeset not found no need to go for changefiles
-
             if changeset in processed_changesets:
                 collect_changefile_stats(
                     user,
@@ -860,7 +858,6 @@ def main():
         )
         sys.exit()
     if (end_date - start_date).days > 1:
-
         if args.hashtags:
             if not args.force:
                 print(
@@ -875,7 +872,6 @@ def main():
     print(f"Supplied start_date: {start_date} and end_date: {end_date}")
 
     if args.hashtags or args.changeset:
-
         Changeset = ChangesetToolKit()
         (
             changeset_download_urls,
@@ -1075,7 +1071,17 @@ def main():
         if args.tm_stats:
             print("Generating TM Stats ....")
             # apply function to create new column with list of projects
+            columns_to_drop = [
+                "tm_mapping_level",
+                "tasks_mapped",
+                "tasks_validated",
+                "tasks_total",
+                "tm_projects",
+            ]
+            df = df.drop(columns=[col for col in columns_to_drop if col in df.columns])
+
             df1 = df
+
             df1["tm_projects"] = df1["hashtags"].apply(extract_projects)
             unique_projects = list(
                 set(
@@ -1087,29 +1093,30 @@ def main():
             usernames_unique = df1["name"].unique().tolist()
 
             df2 = generate_tm_stats(unique_projects, usernames_unique)
-            # explode projects column to create new row for each project
-            df1 = df1.explode("tm_projects")
+            if not df2.empty:
+                # explode projects column to create new row for each project
+                df1 = df1.explode("tm_projects")
 
-            # merge df1 and df2_grouped on username and project
-            df_merged = pd.merge(df1, df2, on=["name", "tm_projects"], how="left")
+                # merge df1 and df2_grouped on username and project
+                df_merged = pd.merge(df1, df2, on=["name", "tm_projects"], how="left")
 
-            # group df_merged by username to combine information for each unique username
-            df_final = (
-                df_merged.groupby(["name"])
-                .agg(
-                    {
-                        "tm_mapping_level": "first",
-                        "tasks_mapped": "sum",
-                        "tasks_validated": "sum",
-                        "tasks_total": "sum",
-                    }
+                # group df_merged by username to combine information for each unique username
+                df_final = (
+                    df_merged.groupby(["name"])
+                    .agg(
+                        {
+                            "tm_mapping_level": "first",
+                            "tasks_mapped": "sum",
+                            "tasks_validated": "sum",
+                            "tasks_total": "sum",
+                        }
+                    )
+                    .reset_index()
                 )
-                .reset_index()
-            )
 
-            df = pd.merge(df, df_final, on=["name"], how="left")
-            df["tm_projects"] = df["tm_projects"].apply(lambda x: ",".join(x))
-            print(df)
+                df = pd.merge(df, df_final, on=["name"], how="left")
+                df["tm_projects"] = df["tm_projects"].apply(lambda x: ",".join(x))
+                print(df)
 
         if args.summary:
             summary_df = pd.json_normalize(list(summary_interval.values()))
@@ -1261,6 +1268,27 @@ def main():
                 for i in range(1, 6 if len(df) > 6 else len(df)):
                     top_users += f"- {df.loc[i, 'name']} : {humanize.intword(df.loc[i, 'map_changes'])} Map Changes\n"
                 file.write(top_users)
+                if args.tm_stats:
+                    top_tm_mappers = "\nTop 5 Tasking Manager Mappers are : \n"
+                    tm_df = df
+                    tm_sort = tm_df.sort_values(by="tasks_mapped", ascending=False)
+
+                    # Get the top 5 rows
+                    tm_top_5 = tm_sort.head(5)
+                    for index, row in tm_top_5.iterrows():
+                        name = row["name"]
+                        tasks_mapped = row["tasks_mapped"]
+                        top_tm_mappers += f"- {name} : {humanize.intword(tasks_mapped)} Tasks Mapped\n"
+                    file.write(top_tm_mappers)
+
+                    top_tm_validators = "\nTop 5 Tasking Manager Validators are : \n"
+                    tm_sort = tm_df.sort_values(by="tasks_validated", ascending=False)
+                    tm_top_5 = tm_sort.head(5)
+                    for index, row in tm_top_5.iterrows():
+                        name = row["name"]
+                        tasks_validated = row["tasks_validated"]
+                        top_tm_validators += f"- {name} : {humanize.intword(tasks_validated)} Tasks Validated\n"
+                    file.write(top_tm_validators)
 
                 user_tags_summary = "\nSummary of Supplied Tags\n"
                 user_tag = "poi"
